// MIS3502 - Web Service Template
// Created by: Jeremy Shafer
// Fall 2023

// REMINDER - Don't forget to change your database connection
// timeout from 3 seconds to 3 minutes.
// Look under Configuration / General Configuration

// declarations *****************************************
const qs = require('qs'); //for parsing URL encoded data
const axios = require('axios'); // for calling another API
const base64 = require('js-base64'); // encode / decode
const mysql = require('mysql2/promise');  //for talking to a database

const dboptions = {
  'user' : '',
  'password' : '',
  'database' : '',
  'host' : 'dataanalytics.temple.edu'
};

//global connection variable
var connection;

const features = [
`Issue a GET against ./auth and provide a username and password. The API will respond with a JSON object containing all the user information except the password.`,
`Issue a POST against ./users to create a new user record. You must provide the following keys: username, password, email. The API will respond with the userid of the newly created record.`,
`Issue a GET against ./scancheck to check an event code. You must provide the following keys: eventcode, userid. The eventcode is a UUID. It is a string 36 characters in length. If the event code is good, the API will return a JSON object containing the title of the event, the date and time of the history event expressed as text, and the same date and time expressed as epoch time.`,
`Issue a GET against ./history to retrieve a JSON object containing record of all the events attended by the user. You must provide the following key: userid. The events in the repsonse will be ordered by thier creation date/time with the most resent attendance records being listed first.`,
'This API created by Jeremy Shafer'
]


// supporting functions *********************************


let getScancheck = async (res, query) => {
	
	let eventcode = query.eventcode;
	let userid = query.userid;
	
	// testing for the lenght of 36 in the event code is very good. 
	// it is a sign that the student is thinking a bit more than the rest.
	//
	if (eventcode == undefined || eventcode == "" || eventcode.length != 36){
		return formatres(res,"The key eventcode is missing or incorrect.",400);
	}
	if (userid == undefined || userid == "" || isNaN(userid)){
		return formatres(res,"The key userid is missing or incorrect.",400);
	}
	
	let txtSQL = "select eventname, eventid from events where eventcode = ?";
	let [result] = await connection.execute(txtSQL,[eventcode]);
	
	
	//superior solutions will have this step
	if (result.length == 0){
		return formatres(res,"Invalid Event Code.",400);	
	}

	let eventname = result[0].eventname;
	let eventid = result[0].eventid;

	
	//-- step 2
	let txtSQL2 = "insert into history (eventname,eventid,userid) values (?,?,?)"
	let [result2] = await connection.execute(txtSQL2,[eventname,eventid,userid]);
	
	let historyid = result2.insertId;
	
	//-- step 3
	let txtSQL3 = "select eventname, DATE_FORMAT(datetime, '%d/%m/%y %T') as datetime , UNIX_TIMESTAMP(datetime) as epochtime from history where historyid = ?"
	let [result3] = await connection.execute(txtSQL3,[historyid]);
	return formatres(res,result3,200);
}


let getHistory = async (res, query) => {
	let userid = query.userid;
	
	if (userid == undefined || userid == "" || isNaN(userid)){
		return formatres(res,"The key userid is missing or incorrect.",400);
	}
	
	let txtSQL = "select historyid, eventname, DATE_FORMAT(datetime, '%d/%m/%y %T') as datetime , UNIX_TIMESTAMP(datetime) as epochtime from history where userid = ? order by historyid desc";
	let [result] = await connection.execute(txtSQL,[userid]);
	return formatres(res,result,200);
}


let getAuth = async (res,query) => {

	// get the expected variable
	let username = query.username;
	let password = query.password;
	
	//error trap
	if (username == undefined || username == "" ){
		return formatres(res,"The key username is missing or incorrect.",400);
	}
	if (password == undefined || password == "" ){
		return formatres(res,"The key password is missing or incorrect.",400);
	}
		
	//work and return the result
	let txtSQL = "select userid, username, email, " + 
		" sms, datetime, lastlogin, usertype, firstname, lastname " +
		" from users where username = ? AND password = ? ";
		
	let [result] = await connection.execute(txtSQL,[username , password ]);
	return formatres(res,result,200);
}

// do not delete this handy little supporting function
let formatres = (res, output, statusCode) => {
	
	// kill the global database connection
	if (connection != undefined &&  
		typeof(connection)=='object' &&  
		typeof(connection.end())=='object'  ){
		connection.end();
	}

	res.statusCode = statusCode;
	res.body = JSON.stringify(output);
	return res;	
}

// do not delete this handy little supportng function
function isEmpty(obj) {
    return Object.keys(obj).length === 0;
}

// My Routing Function **********************************

let myRoutingFunction = (res,method,path,query,body) => {

	// conditional statements go here.
	// look at the path and method and return the output from the 
	// correct supporting function.

	// Simple GET request with no features specified results
	// in a list of features / instructions
	if (method == "GET" && path == ""){
		return formatres(res, features, 200);
	}

	// working on feature 4
	if (method == "GET" && path == "history"){
		return getHistory(res,query);
	}

	// working on feature 3
	if (method == "GET" && path == "scancheck"){
		return getScancheck(res,query);
	}
	
	// working on feature 2
	if (method == "POST" && path == "users"){
		return formatres("This feature is under construction.",200);
		//return postUser(res,body);
	}

	// working on feature 1
	if (method == "GET" && path == "auth"){
		return getAuth(res,query);
	}

	return(res);
}


// HTTP event handler ****************************************

// Students should not have to change the code here.
// Students should be able to read and understand the code here.

exports.handler = async (request) => {

	connection = await mysql.createConnection(dboptions);	

	// identify the method (it will be a string)
	let method = request["httpMethod"];
	
	// identify the path (it will also be a string)
	let fullpath = request["path"];
	
	// we clean the full path up a little bit
	if (fullpath == undefined || fullpath == null){ fullpath = ""};
	let pathitems = fullpath.split("/");
	let path = pathitems[2];
	if (path == undefined || path == null){ path = ""};
	
	// identify the querystring ( we will convert it to 
	//   a JSON object named query)
	let query = request["queryStringParameters"];
	if (query == undefined || query == null){ query={} };
	
	// identify the body (we will convert it to 
	//   a JSON object named body)
	let body = qs.parse(request["body"]);
	if (body == undefined || body == null){ body={} };

	// Create the default response object that will include 
	// the status code, the headers needed by CORS, and
	// the string to be returned formatted as a JSON data structure.
    let res = {
        'statusCode': 400,
        'headers': {
            'Access-Control-Allow-Origin': '*',
            'Access-Control-Allow-Credentials': true  
        },
        'body': JSON.stringify("Feature not found."),
    };

	// run all the parameters through my routing function
	// and return the result
    return myRoutingFunction(res,method,path,query,body);
    
    //*** this is a good place to test one supporting function at a time
    //body={"username":"fred","password":"123456","email":"freddy@test.com"};
    //return postUser(res,body);
    
};